#### Run to create MySql container from docker hub
- `docker run --name mysql-basic \
    -e MYSQL_USER=user1 \
	-e MYSQL_PASSWORD=mypa55 \
    -e MYSQL_DATABASE=items \
	-e MYSQL_ROOT_PASSWORD=r00tpa55 \
    -p 30306:3306 mysql:latest`

- `docker run --name mysql-basic -e MYSQL_USER=user1 -e MYSQL_PASSWORD=mypa55 -e MYSQL_DATABASE=items -e MYSQL_ROOT_PASSWORD=r00tpa55 -d mysql:5.6`

- Access the container sandbox by running the following command:
	- `docker exec -it mysql-basic bash`

- Log in to MySQL Container as the database administrator user (root).Run the following command from the container terminal to connect to the database:
	- `mysql -pr00tpa55`
- Create a new table in the items database. From the MySQL prompt, run the following command to access the database:
	- `use items;`
- From the MySQL prompt, create a table called Projects in the items database:
	- `CREATE TABLE Courses (id int NOT NULL, name varchar(255) NOT NULL, PRIMARY KEY (id));`
- Run the following command to verify that the table was created:
	- `show tables;`
- Insert a row in the table by running the following command:
	- `insert into Courses (id, name) values (1, 'DO081x');`
- Run the following command to verify that the project information was added to the table:
	- `select * from Courses;`

### _Run Mysql and Connect Host and Docker_
- Get the ip of the host
	- `hostname -I`
- Connect Mysql to host
	- `mysql -h Host_IP -P Host_Port -u User_Name -p User_Password Database_Name`
	- Example: `mysql -h 192.168.1.101 -P 30306 -uuser1 -pmypa55 Items`
- Run a script to create tables
	- `source ~/path/db.sql`
- Check in the console
	- `show tables;`
	- `select * from Items;` (or run `select` query to check)
- Go to the docker mysql
	- `docker exec -it Image_Name bash`
	- Example: `docker exec -it mysql-local bash`
- Connect mysql in docker image
	- `mysql -h $HOSTNAME -uuser1 -pmypa55 Items`
- Run same query to check in docker image
	- `show tables;`
	- `select * from Items;` (It should give same result as host)

### _Preserved data generated by container processes accross multiple instances of container_
- Container storage is said to be ephemeral means its contents are not preserved after the container is stopped. Containerized applications are supposed to work on the assumption that they will always start with an empty storage.
- _Configure a host folder for container use_
	- `mkdir /var/DB_files` (create a folder in host or remote network)
	- Example: `mkdir -p /var/local/mysql`
	- `chmod o+rwx /var/DB_files` (change the permission of the folder to access by container)
	- Example: `sudo chmod o+rwx /var/local/mysql`
	- `chcon -t svirt_sandbox_file_t /var/DB_files` (apply the appropriate SELinux context to the directory)(this for RHEL)
	- Example: `sudo chcon -t svirt_sandbox_file_t /var/local/mysql`
- Mount host folder with docker image folder using `-v` option
	- `docker run -d --name IMAGE_NAME -e MYSQL_DATABASE=DATABASE_NAME -e MYSQL_USER=USER_NAME -e MYSQL_PASSWORD=USER_PASSWORD -e MYSQL_ROOT_PASSWORD=MYSQL_ROOT_PASSWORD -p HOST_PORT:CONTAINER_PORT -v HOST_PATH_DB_FILES:IMAGE_PATH_DB_FILES CONTAINER_NAME:TAG_NAME` (TAG_NAME is optional)
	- Example: `docker run -d --name mysql-local -e MYSQL_DATABASE=Items -e MYSQL_USER=user1 -e MYSQL_PASSWORD=mypa55 -e MYSQL_ROOT_PASSWORD=r00tpa55 -p 30306:3306 -v /var/local/mysql:/var/lib/mysql mysql:latest`